local HotCandy = LibStub("AceAddon-3.0"):NewAddon("HotCandy", "AceEvent-3.0", "AceBucket-3.0", "AceConsole-3.0")

local VERSION = tonumber(("$Revision: 66050 $"):match("%d+"))
HotCandy.revision = VERSION
HotCandy.versionstring = "1.1 |cffff8888r%d|r"
HotCandy.version = HotCandy.versionstring:format(VERSION)
HotCandy.date = ("$Date: 2008-03-26 08:32:53 -0400 (Wed, 26 Mar 2008) $"):match("%d%d%d%d%-%d%d%-%d%d")

local media = LibStub("LibSharedMedia-3.0")

local itemSets = {
	["Stormrage"] = { 16897, 16898, 16899, 16900, 16901, 16902, 16903, 16904 },
	["Nordrassil"] = { 30216, 30217, 30219, 30220, 30221 },
	["Transcendence"] = { 16919, 16920, 16921, 16922, 16923, 16924, 16925, 16926 },
	["Avatar Raiment"] = { 30150, 30151, 30152, 30153, 30154 },
}
	-- spell = duration
local durations = {
	[774] = 12, -- Rejuvenation
	[8936] = 21, -- Regrowth
	[139] = 15, -- Renew
	[2060] = 0, -- Greater Heal
	[33763] = 7, -- Lifebloom
	[28880] = 15, -- Gift of the Naaru
}

local spell = {}
local track = {}

local defaults = {
	profile = {
		growup = false,
		texture = "BantoBar",
		width = 180,
		scale = 1,
		stackfirst = true,
		noname = false,
	}
}

function HotCandy:OnInitialize()
	AceLibrary("CandyBar-2.0"):embed(self)
	self.db = LibStub("AceDB-3.0"):New("HotCandyDB", defaults, "Default")
	
	media:Register("statusbar", "BantoBar", "Interface\\Addons\\HotCandy\\textures\\default")
end

function HotCandy:OnEnable()
	self:SetupFrames()
	
	self:RegisterEvent("UNIT_SPELLCAST_SENT", "SpellCastEvent")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "SpellCastEvent")
	self:RegisterBucketEvent("UNIT_INVENTORY_CHANGED", 1, "equipChanged")
	
	self:UpdateSetBonus()
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HotCandy", self.CreateOptionsTable)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HotCandy")
end

function HotCandy:CreateOptionsTable()
	self = HotCandy
	if not self.options then
		local optGet = function(info) return self.db.profile[info.arg or info[#info]] end
		local optSet = function(info, value) self.db.profile[info.arg or info[#info]] = value end
		
		local lsmGet = function() 
			local list = media:List("statusbar")
			local media = self.db.profile.texture
			for i=1,#list do
				if list[i] == media then return i end
			end
			return false
		end
		
		local lsmSet = function(info, value)
			self.db.profile.texture = media:List("statusbar")[value]
		end
		
		self.options = {
			name = "HotCandy",
			type = "group",
			get = optGet,
			set = optSet,
			handler = self,
			args = {
				intro = {
					cmdHidden = true,
					order = 1,
					type = "description",
					name = "You can control the behaviour and the style of HotCandy using the settings below.",
				},
				growup = {
					order = 10,
					name = "Grow Up",
					desc = "Toggle grow direction of the bars",
					type = "toggle",
					width = "full",
				},
				texture = {
					order = 20,
					type = "select",
					name = "Texture",
					desc = "Set the texture for the bars.",
					values = media:List("statusbar"),
					get = lsmGet,
					set = lsmSet,
				},
				br1 = {
					cmdHidden = true,
					type = "description",
					order = 29,
					name = "",
				},
				width = {
					order = 30,
					type = "range",
					name = "Bar Width",
					desc = "Set the width of the bars.",
					min = 50, max = 500, step = 1,
				},
				scale = {
					order = 31,
					type = "range",
					name = "Bar Scale",
					desc = "Set the scale of the bars.",
					min = .1, max = 2, step = 0.01,
					isPercent = true,
				},
				format = {
					order = 40,
					type = "group",
					name = "Text Format",
					desc = "Configure the format of the text on the timer bar.",
					dialogInline = true,
					args = {
						noname = {
							type = "toggle",
							name = "No HoT name",
							desc = "Do not show the name of the HoT.",
						},
						stackfirst = {
							type = "toggle",
							name = "Stack First",
							desc = "Show HoT stack count before the name.",
						},
					},
				},
				anchordesc = {
					cmdHidden = true,
					order = 99,
					type = "description",
					name = "To test your settings and to move the bars, you can show the anchor on your screen, and once you're done setting up, simply close it again.",
				},
				anchor = {
					order = 100,
					name = "Show Anchor",
					desc = "Show the Anchor for moving the bars",
					type = "execute",
					func = "ShowAnchors",
				},
			}
		}
	end
	
	return self.options
end

function HotCandy:UpdateSetBonus()
	local _, class = UnitClass("player")
	
	for k,v in pairs(durations) do
		local name, _, icon = GetSpellInfo(k)
		if not track[name] then
			track[name] = { icon = icon }
		end
		track[name].duration = v
	end
	
	if class == "DRUID" then
		if (self:getWornSetPieces("Stormrage") >= 8) then
			local key = (GetSpellInfo(774))
			track[key].duration = track[key].duration + 3
		end
		if (self:getWornSetPieces("Nordrassil") >= 2) then
			local key = (GetSpellInfo(8936))
			track[key].duration = track[key].duration + 6
		end
	elseif class == "PRIEST" then
		if (self:getWornSetPieces("Transcendence") >= 8) then
			local key = (GetSpellInfo(2060))
			track[key].duration = track[key].duration + 15
		end
		if (self:getWornSetPieces("Avatar Raiment") >= 4) then
			local key = (GetSpellInfo(139))
			track[key].duration = track[key].duration + 3
		end
	end
end

-- taken from Threat-1.0
function HotCandy:getWornSetPieces(name)
	local ct = 0
	local data = itemSets[name]
	if data then
		for _, itemID in ipairs(data) do
			if IsEquippedItem( itemID ) then
				ct = ct + 1
			end
		end
	end
	return ct
end

function HotCandy:equipChanged(units)
	if (units and not units.player) or InCombatLockdown() then
		return
	end
	self:UpdateSetBonus()
end

function HotCandy:SpellCastEvent(event, unit, spellName, rank, target)
	if ( event == "UNIT_SPELLCAST_SENT" ) then
		spell['spell'] = spellName
		spell['target'] = target
	elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		if spell['spell'] == spellName then
			self:FireSpell(spellName, spell['target'])
		end
		spell['spell'] = nil
	end
end

local fmt = string.format

local lb = (GetSpellInfo(33763))
function HotCandy:FireSpell(spellName, spellTarget)
	local spell = track[spellName]
	if spell and spell.duration > 0 then
		local id = spellName..spellTarget
		local stack = ""
		if spellName == lb then
			local i = 1
			local buff, _, _, count, duration = UnitBuff(spellTarget, i)
			while buff and duration and buff ~= spellName do
				i = i + 1
				buff, _, _, count, duration = UnitBuff(spellTarget, i)
			end
			if count and count > 0 then
				count = min(count + 1, 3)
				stack = fmt("(%d)", count)
			end
		end
		local text
		if not self.db.profile.noname then
			if self.db.profile.stackfirst then
				text = stack .. spellName .. " - " .. spellTarget
			else
				text = spellName .. stack .." - " .. spellTarget
			end
		else
			text = stack .. spellTarget
		end
		self:ShowCandyBar(id, text, spell.duration, spell.icon, 0, 1, 0, 1, 1, 0, 1, 0.5, 0, 1, 0, 0)
	end
end

function HotCandy:ShowCandyBar(id, text, time, icon, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10)
	id = "HotCandy"..id
	local u = self.db.profile.growup
	self:RegisterCandyBarGroup("HotCandyGroup")
	self:SetCandyBarGroupPoint("HotCandyGroup", u and "BOTTOM" or "TOP", self.frames.anchor, u and "TOP" or "BOTTOM", 0, 0)
	self:SetCandyBarGroupGrowth("HotCandyGroup", u)
	
	self:RegisterCandyBar(id, time, text, icon, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10)
	self:RegisterCandyBarWithGroup(id, "HotCandyGroup")
	self:SetCandyBarTexture(id, media:Fetch("statusbar", self.db.profile.texture))
	self:SetCandyBarFade(id, 5)
  self:SetCandyBarWidth(id, self.db.profile.width)
	self:SetCandyBarScale(id, self.db.profile.scale)
	self:StartCandyBar(id, true)
end
